#!/usr/bin/perl
#
## Format variable output columns into Tucnak 'C_W file' spacing
## Copyleft © 2016 Dave Gilligan.  ## <g1ogy_|at|_g1ogy.com>
#
##############################################################################
## This program is free software; you can redistribute it and/or modify it   
## under the terms of the GNU General Public License as published by the 
## Free Software Foundation; either version 2 of the License, or (at your    
## option) any later version. 
## 
## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
## See the GNU General Public License for more details. 
##   http://www.m1cro.org.uk/Members/G1OGY/GNU-GPL.html
##
##     Succinctly: If it breaks - you get to keep all the pieces.
##
##############################################################################

use Getopt::Long;
# use warnings;

$prog = "TucformatC_W";
$version = "V 0.01";
$datestring="[20161120]";
$auth = "Author:G1OGY <g1ogy!at!g1ogy.com> ";
$arights="Licenced under GNU GPL..";

# define Usage

    die "\n\"$prog\"  $version           $datestring     $auth
        $arights
    Usage:
      \"$prog\"  inputfile.ext  [outputfile.ext]  [[-d|--date] datestring]

      \"$prog\" produces Tucnak format output.  
        an arbitary list is expected for input.  Should you wish to
        convert TACLog C_W format (TEXT) files for use with Tucnak, 
        please use the companion program \"tac2tuc.pl\".

        \"$prog\" requires an INPUT file name argument and optionally, 
        an OUTPUT file name.
        A raw column format of: 
        CALL  LOCATOR 
        is mandatory; additional .C_W file fields are expected, separated
        by \"WhiteSpace\".  Date, if present, MUST be the Tucnak way: YYYYMMDD
        Input data may be mixed.  ie:
        CALL    LOCATOR     DATE
        CALL LOCATOR
        (the amount of \"WhiteSpace\" is not important).
        Full duplicate lines are removed and although unnecessary fir Tucnak
        the output is sorted logically, so that /-modified calls follow `rawcalls`.
        A Date, if present, will be retained; but see `-d` option below.
                        
        By appending the flag `-d|--date` followed by a (Tucnak) Date : 20161111
        a specific date can be ~forced~ into ALL the output.
        Alternatively, if Date is missing in the input file and `-d|--date` is not
        specified, <TODAY> will be entered where relevant, automatically.

        NO blank lines nor Column Headers allowed!

        Default OUTPUT file is \"TUC-C_W.TXT\" in the `current` directory.\n"
    unless @ARGV;

# define local variables

my ($rawfile)=$ARGV[0];
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=gmtime(time);
my ($monno)="";
# sort the silly single-numeric month 
    MonManip();
my ($today)=($year += 1900).($monno).$mday;
my (%seen)=();

# main 

GetOptions('d|date=s');

# interprocess file
	open (TMP_FILE,">tmpfile") || die "\nCannot Create Temporary Workspace!\n";
# final output file
    if ($ARGV[1] ne "") {
		$outfile = $ARGV[1];
    }
        else { 
		$outfile = "TUC-C_W.TXT";
        }
# input file
    open (RAWLIST, "$rawfile") || die "\nInput File not Found\! Check the path : \n$!";

    print "\nWorking...\n";
#
# main
    while (<RAWLIST>) {
# read the input and write a delimited output to an array	
        chomp;
        ($call,$loc,$qsodate) = split;
# no date - add one
        unless ($qsodate) {     
        	$qsodate="$today"; 
        }
# add a delimiter - make life easier later    
 	    push (@returndata, ($call.";".$loc.";".$qsodate));     
    }	
   
# dedupe  

	foreach $callrecord (@returndata) {
   	    push (@uniq, $callrecord) unless $seen {$callrecord} ++;
	}
# no need to sort (lexically) here as done (logically) using the O/S sort

# break out into individual vars again to write the correctly formatted output file
      	foreach $record (@uniq) {
     	    ($call,$loc,$qsodate) = split (/;/,$record);
# overwrite date if forced on command line
	    if ($opt_d)	{
		$qsodate = $opt_d;
	    }
       	        write (TMP_FILE);
        }		

# sort (logically) ~externally~ in the O/S [Linux and Windows act the same]
		system("sort tmpfile > $outfile");
	
# clean up 
    close (RAWLIST); close (TMP_FILE);
    system("rm tmpfile") ; 


# output format 

format TMP_FILE =
@<<<<<<<<<<<<<<@<<<<<<@<<<<<<<
$call,$loc,$qsodate
.

# end_of_format
     credits();

############################################################################################
# Month manipulation

sub MonManip {
    $mon=$mon+=1;
    	 if ($mon < 10)	{
            $monno = "0".$mon;
	 }
         else   {
             $monno=$mon
         }    
}

############################################################################################
# Credits

sub credits {
    print ("\n\n\"$prog\" $version         $datestring       $auth\n\n");
        print ("Credits: ");
        print ("$prog is made available under the terms of the GNU General Public Licence.\n");
    }


##########

__END__

